#ifndef HW_LIB_TFT_H
#define HW_LIB_TFT_H
#ifdef __cplusplus
extern "C" {
#endif
#include "stdint.h"
#include "stdlib.h"


#define REFRESH_CALL_ENABLE 1 //使用DMA或者整体刷新函数
#define HZK_FONT              //使用HZK 12/16 字体 tools下可自由生成
#define UTF8_TO_UNICODE       //启用UTF8转换显示
//#define LVGL_FONT           //启用LVGL字体

#define TFT_ST7735


typedef struct TFT_Dev TFT_T;

/**
 * @brief   TFT命令处理函数指针类型
 * @param   data: [输入] 数据指针
 * @param   len: [输入] 数据长度
 * @return  uint8_t 返回值
 */
typedef uint8_t (*TFT_CMD_t)(uint8_t *data, size_t len);

/**
 * @brief   TFT数据处理函数指针类型
 * @param   data: [输入] 数据指针
 * @param   len: [输入] 数据长度
 * @return  uint8_t 返回值
 */
typedef uint8_t (*TFT_DATA_t)(uint8_t *data, size_t len);

/**
 * @brief   TFT数据读取函数指针类型
 * @param   data: [输出] 数据指针
 * @param   len: [输入] 数据长度
 * @return  uint8_t 返回值
 */
typedef uint8_t (*TFT_READ_t)(uint8_t *data, size_t len);

/**
 * @brief   TFT背光控制函数指针类型
 * @param   data: [输入] 数据
 * @return  uint8_t 返回值
 */
typedef uint8_t (*TFT_BLACKLIGHT_t)(uint8_t data);

/**
 * @brief   TFT延迟函数指针类型
 * @param   ms: [输入] 毫秒延迟
 * @return  uint8_t 返回值
 */
typedef uint8_t (*TFT_DELAY_t)(uint32_t ms);

#if REFRESH_CALL_ENABLE

/**
 * @brief   TFT 刷新函数指针类型
 */
typedef void (*TFT_REFRESH_t)(TFT_T *dev);

#endif

/**
* @brief   TFT状态枚举
*/
typedef enum {
    IDLE, /**< 空闲状态 */
    WRITE, /**< 写入状态 */
    REFRESH, /**< 刷新状态 */
} TFT_STATE_T;

typedef enum {
    LONGITUDINAL,/* 液晶屏纵向选择 */
    LONGITUDINAL_180,/* 液晶屏纵向旋转 180° 方向选择 */
    HORIZONTAL,/* 液晶屏横向选择 */
    HORIZONTAL_180,/* 液晶屏横向旋转 180° 方向选择 */
} TFT_DIR_T;

typedef union TFT_Color {
    uint16_t color;
    uint16_t u16;
    uint8_t u8[2];
} TFT_Color_t;

/**
 * @brief   TFT设备结构体
 */
struct TFT_Dev {
    TFT_DIR_T dir: 4; /**< 显示方向 */
    TFT_STATE_T state: 4; /**< TFT状态 */
    uint16_t width; /**< 显示宽度 */
    uint16_t height; /**< 显示高度 */
    TFT_CMD_t cmd; /**< TFT命令处理函数指针 */
    TFT_DATA_t data; /**< TFT数据处理函数指针 */
    TFT_READ_t read; /**< TFT数据读取函数指针 */
    TFT_BLACKLIGHT_t blacklight; /**< TFT背光控制函数指针 */
    TFT_DELAY_t delay; /**< TFT延迟函数指针类型 */
#if REFRESH_CALL_ENABLE
    TFT_REFRESH_t call;    /**< TFT刷新函数指针 */
#endif
};

extern TFT_Color_t POINT_COLOR;
extern TFT_Color_t BACK_COLOR;

void TFT_Init(TFT_T dev);

uint32_t TFT_ReadID(TFT_T dev);

void TFT_SetDir(TFT_T dev, TFT_DIR_T dir);


#ifdef LVGL_FONT

#include "lvgl_font.h"

/**
 * @brief   在TFT上显示字符串
 * @param   dev: [输入] TFT设备结构体指针
 * @param   font: [输入] 字体信息
 * @param   s: [输入] 要显示的字符串
 * @param   x: [输入] 字符串左上角 x 坐标
 * @param   y: [输入] 字符串左上角 y 坐标
 * @return  void
 * @example TFT_DisplayString(&oled_device, &font_arial_16, "Hello World", 10, 20);
**/
void TFT_DisplayString(TFT_T *dev, const lv_font_t *font, uint8_t *s, uint16_t x, uint16_t y);

#endif
#ifdef __cplusplus
}
#endif
#endif //HW_LIB_TFT_H
